/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 *
 * @format
 */

module.exports = {
  setupFiles: ["./jest.setup.js", "./tests/tools.js"],
  testRegex: "/tests/.*\\.test\\.js$",
  watchman: false,
};
